==> llm-docs-0.1.2.tgz: package/docs/drupal/integration.md <==
---
id: drupal-integration
---
# Drupal Integration

<!-- title: Drupal Integration -->
<!-- description: Developer documentation for integrating Drupal with the Bluefly Platform -->
<!-- lastUpdated: 2024-03-19 -->

This guide covers the integration between Drupal and the Bluefly Platform, including module development, API integration, custom workflows, and testing strategies.

## Architecture

### Core Components

```php
// Bluefly Service Interface
interface BlueflyServiceInterface {
  public function getAgent(string $name): AgentInterface;
  public function executeTask(string $agent, array $task): array;
  public function getStatus(string $taskId): array;
  public function registerWebhook(string $url, array $events): void;
}

// Agent Interface
interface AgentInterface {
  public function getName(): string;
  public function getCapabilities(): array;
  public function execute(array $input): array;
  public function learn(array $experience): void;
}

// Task Definition
class Task {
  public string $id;
  public string $type;
  public array $input;
  public array $context;
  public array $constraints;
  public int $priority;
  
  public function validate(): void;
  public function toArray(): array;
}
```

### Directory Structure

```
modules/custom/bluefly/
├── bluefly.info.yml           # Module info
├── bluefly.module            # Module file
├── bluefly.services.yml      # Service definitions
├── bluefly.routing.yml       # Route definitions
├── bluefly.permissions.yml   # Permission definitions
├── src/
│   ├── Controller/          # Controllers
│   │   ├── Block/         # Block plugins
│   │   ├── Field/         # Field plugins
│   │   └── Action/        # Action plugins
│   ├── Entity/            # Entity types
│   ├── Form/              # Forms
│   └── EventSubscriber/   # Event subscribers
├── config/
│   ├── install/           # Default config
│   └── schema/            # Config schema
└── tests/
    ├── Kernel/            # Kernel tests
    └── Functional/        # Functional tests
```

## Development

### Module Setup

1. Create module structure:

```yaml
# bluefly.info.yml
name: Bluefly Integration
type: module
description: 'Integrates Drupal with the Bluefly Platform'
package: Bluefly
core_version_requirement: ^9 || ^10
dependencies:
  - drupal:jsonapi
  - drupal:rest
```

2. Define services:

```yaml
# bluefly.services.yml
services:
  bluefly.service:
    class: Drupal\bluefly\Service\BlueflyService
    arguments: ['@http_client', '@config.factory']
    tags:
      - { name: service_collector, tag: 'bluefly.agent', call: addAgent }
  
  bluefly.agent_registry:
    class: Drupal\bluefly\Service\AgentRegistry
    arguments: ['@bluefly.service']
```

3. Define routes:

```yaml
# bluefly.routing.yml
bluefly.task.execute:
  path: '/api/bluefly/task/execute'
  defaults:
    _controller: '\Drupal\bluefly\Controller\TaskController::execute'
    _title: 'Execute Bluefly Task'
  requirements:
    _permission: 'execute bluefly tasks'
  methods: [POST]
  options:
    _auth: ['basic_auth', 'cookie']
```

### Service Implementation

1. Create the service:

```php
// src/Service/BlueflyService.php
namespace Drupal\bluefly\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use GuzzleHttp\ClientInterface;
use Drupal\bluefly\AgentInterface;

class BlueflyService implements BlueflyServiceInterface {
  private ClientInterface $httpClient;
  private ConfigFactoryInterface $configFactory;
  private array $agents = [];
  
  public function __construct(
    ClientInterface $httpClient,
    ConfigFactoryInterface $configFactory
  ) {
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
  }
  
  public function getAgent(string $name): AgentInterface {
    if (!isset($this->agents[$name])) {
      throw new \InvalidArgumentException("Agent $name not found");
    }
    return $this->agents[$name];
  }
  
  public function executeTask(string $agent, array $task): array {
    $agent = $this->getAgent($agent);
    $task = new Task($task);
    $task->validate();
    
    try {
      $result = $agent->execute($task->toArray());
      $this->logTask($task, $result);
      return $result;
    } catch (\Exception $e) {
      $this->handleError($e, $task);
      throw $e;
    }
  }
  
  public function addAgent(AgentInterface $agent): void {
    $this->agents[$agent->getName()] = $agent;
  }
  
  private function logTask(Task $task, array $result): void {
    \Drupal::logger('bluefly')->info('Task executed: @task', [
      '@task' => json_encode([
        'id' => $task->id,
        'type' => $task->type,
        'result' => $result
      ])
    ]);
  }
}
```

2. Create an agent:

```php
// src/Plugin/Agent/ContentAgent.php
namespace Drupal\bluefly\Plugin\Agent;

use Drupal\bluefly\AgentInterface;
use Drupal\Core\Plugin\PluginBase;

/**
 * @Agent(
 *   id = "content",
 *   label = @Translation("Content Agent"),
 *   description = @Translation("Handles content-related tasks")
 * )
 */
class ContentAgent extends PluginBase implements AgentInterface {
  public function getName(): string {
    return 'content';
  }
  
  public function getCapabilities(): array {
    return [
      'generate' => [
        'description' => 'Generate content',
        'parameters' => [
          'type' => 'string',
          'prompt' => 'string'
        ]
      ],
      'summarize' => [
        'description' => 'Summarize content',
        'parameters' => [
          'content' => 'string',
          'length' => 'integer'
        ]
      ]
    ];
  }
  
  public function execute(array $input): array {
    $capability = $input['capability'] ?? 'generate';
    
    switch ($capability) {
      case 'generate':
        return $this->generateContent($input);
      case 'summarize':
        return $this->summarizeContent($input);
      default:
        throw new \InvalidArgumentException("Unknown capability: $capability");
    }
  }
  
  private function generateContent(array $input): array {
    // Implement content generation
    $prompt = $input['prompt'];
    $type = $input['type'];
    
    // Call Bluefly API
    $response = $this->callApi('generate', [
      'prompt' => $prompt,
      'type' => $type
    ]);
    
    return [
      'success' => true,
      'content' => $response['content'],
      'metadata' => [
        'model' => $response['model'],
        'tokens' => $response['usage']
      ]
    ];
  }
}
```

### Controller Implementation

```php
// src/Controller/TaskController.php
namespace Drupal\bluefly\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\bluefly\Service\BlueflyServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

class TaskController extends ControllerBase {
  private BlueflyServiceInterface $blueflyService;
  
  public function __construct(BlueflyServiceInterface $blueflyService) {
    $this->blueflyService = $blueflyService;
  }
  
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('bluefly.service')
    );
  }
  
  public function execute(Request $request): JsonResponse {
    try {
      $content = json_decode($request->getContent(), true);
      
      if (!isset($content['agent'], $content['task'])) {
        throw new \InvalidArgumentException('Missing required parameters');
      }
      
      $result = $this->blueflyService->executeTask(
        $content['agent'],
        $content['task']
      );
      
      return new JsonResponse($result);
    } catch (\Exception $e) {
      return new JsonResponse([
        'error' => $e->getMessage()
      ], 400);
    }
  }
}
```

### Event Subscribers

```php
// src/EventSubscriber/ContentSubscriber.php
namespace Drupal\bluefly\EventSubscriber;

use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\bluefly\Service\BlueflyServiceInterface;

class ContentSubscriber implements EventSubscriberInterface {
  private BlueflyServiceInterface $blueflyService;
  
  public function __construct(BlueflyServiceInterface $blueflyService) {
    $this->blueflyService = $blueflyService;
  }
  
  public static function getSubscribedEvents(): array {
    return [
      'entity.insert' => 'onEntityInsert',
      'entity.update' => 'onEntityUpdate',
      'entity.delete' => 'onEntityDelete'
    ];
  }
  
  public function onEntityInsert(EntityInterface $entity): void {
    if ($entity->getEntityTypeId() === 'node') {
      $this->processContent($entity, 'insert');
    }
  }
  
  private function processContent(EntityInterface $entity, string $action): void {
    try {
      $this->blueflyService->executeTask('content', [
        'action' => $action,
        'entity' => [
          'type' => $entity->getEntityTypeId(),
          'id' => $entity->id(),
          'data' => $entity->toArray()
        ]
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('bluefly')->error('Error processing content: @error', [
        '@error' => $e->getMessage()
      ]);
    }
  }
}
```

## Testing

### Kernel Tests

```php
// tests/Kernel/BlueflyServiceTest.php
namespace Drupal\Tests\bluefly\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\bluefly\Service\BlueflyService;
use Drupal\bluefly\Plugin\Agent\ContentAgent;

class BlueflyServiceTest extends KernelTestBase {
  protected static $modules = ['bluefly', 'system', 'user'];
  
  private BlueflyService $service;
  
  protected function setUp(): void {
    parent::setUp();
    
    $this->service = $this->container->get('bluefly.service');
    $this->service->addAgent(new ContentAgent([], 'content', []));
  }
  
  public function testExecuteTask(): void {
    $result = $this->service->executeTask('content', [
      'capability' => 'generate',
      'type' => 'article',
      'prompt' => 'Test content'
    ]);
    
    $this->assertTrue($result['success']);
    $this->assertArrayHasKey('content', $result);
    $this->assertArrayHasKey('metadata', $result);
  }
}
```

### Functional Tests

```php
// tests/Functional/TaskControllerTest.php
namespace Drupal\Tests\bluefly\Functional;

use Drupal\Tests\BrowserTestBase;

class TaskControllerTest extends BrowserTestBase {
  protected static $modules = ['bluefly', 'rest', 'basic_auth'];
  
  protected function setUp(): void {
    parent::setUp();
    
    $this->drupalCreateUser(['execute bluefly tasks']);
  }
  
  public function testExecuteTask(): void {
    $this->drupalLogin($this->createUser(['execute bluefly tasks']));
    
    $response = $this->drupalPost('/api/bluefly/task/execute', 'application/json', [
      'agent' => 'content',
      'task' => [
        'capability' => 'generate',
        'type' => 'article',
        'prompt' => 'Test content'
      ]
    ]);
    
    $this->assertSession()->statusCodeEquals(200);
    $this->assertJson($response);
    
    $data = json_decode($response, true);
    $this->assertTrue($data['success']);
  }
}
```

## Best Practices

1. **Service Design**
   - Use dependency injection
   - Implement interfaces
   - Handle errors gracefully
   - Log important events
   - Cache when appropriate

2. **Agent Development**
   - Define clear capabilities
   - Validate inputs
   - Handle errors
   - Implement monitoring
   - Document behavior

3. **Security**
   - Validate permissions
   - Sanitize inputs
   - Use CSRF protection
   - Implement rate limiting
   - Log security events

4. **Performance**
   - Use caching
   - Optimize database queries
   - Implement batch processing
   - Monitor resource usage
   - Profile critical paths

## Related Documentation

- [Module Development](../development/modules)
- [API Integration](../development/api)
- [Testing Guide](../development/testing)
- [Security Guide](../operations/security)

==> llm-docs-0.1.4.tgz: package/docs/drupal/integration.md <==
---
id: drupal-integration
---
# Drupal Integration

<!-- title: Drupal Integration -->
<!-- description: Developer documentation for integrating Drupal with the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

This guide covers the integration between Drupal and the LLM Platform Platform, including module development, API integration, custom workflows, and testing strategies.

## Architecture

### Core Components

```php
// LLM Platform Service Interface
interface LLM PlatformServiceInterface {
  public function getAgent(string $name): AgentInterface;
  public function executeTask(string $agent, array $task): array;
  public function getStatus(string $taskId): array;
  public function registerWebhook(string $url, array $events): void;
}

// Agent Interface
interface AgentInterface {
  public function getName(): string;
  public function getCapabilities(): array;
  public function execute(array $input): array;
  public function learn(array $experience): void;
}

// Task Definition
class Task {
  public string $id;
  public string $type;
  public array $input;
  public array $context;
  public array $constraints;
  public int $priority;
  
  public function validate(): void;
  public function toArray(): array;
}
```

### Directory Structure

```
modules/custom/bluefly/
├── bluefly.info.yml           # Module info
├── bluefly.module            # Module file
├── bluefly.services.yml      # Service definitions
├── bluefly.routing.yml       # Route definitions
├── bluefly.permissions.yml   # Permission definitions
├── src/
│   ├── Controller/          # Controllers
│   │   ├── Block/         # Block plugins
│   │   ├── Field/         # Field plugins
│   │   └── Action/        # Action plugins
│   ├── Entity/            # Entity types
│   ├── Form/              # Forms
│   └── EventSubscriber/   # Event subscribers
├── config/
│   ├── install/           # Default config
│   └── schema/            # Config schema
└── tests/
    ├── Kernel/            # Kernel tests
    └── Functional/        # Functional tests
```

## Development

### Module Setup

1. Create module structure:

```yaml
# bluefly.info.yml
name: LLM Platform Integration
type: module
description: 'Integrates Drupal with the LLM Platform Platform'
package: LLM Platform
core_version_requirement: ^9 || ^10
dependencies:
  - drupal:jsonapi
  - drupal:rest
```

2. Define services:

```yaml
# bluefly.services.yml
services:
  bluefly.service:
    class: Drupal\bluefly\Service\LLM PlatformService
    arguments: ['@http_client', '@config.factory']
    tags:
      - { name: service_collector, tag: 'bluefly.agent', call: addAgent }
  
  bluefly.agent_registry:
    class: Drupal\bluefly\Service\AgentRegistry
    arguments: ['@bluefly.service']
```

3. Define routes:

```yaml
# bluefly.routing.yml
bluefly.task.execute:
  path: '/api/bluefly/task/execute'
  defaults:
    _controller: '\Drupal\bluefly\Controller\TaskController::execute'
    _title: 'Execute LLM Platform Task'
  requirements:
    _permission: 'execute bluefly tasks'
  methods: [POST]
  options:
    _auth: ['basic_auth', 'cookie']
```

### Service Implementation

1. Create the service:

```php
// src/Service/LLM PlatformService.php
namespace Drupal\bluefly\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use GuzzleHttp\ClientInterface;
use Drupal\bluefly\AgentInterface;

class LLM PlatformService implements LLM PlatformServiceInterface {
  private ClientInterface $httpClient;
  private ConfigFactoryInterface $configFactory;
  private array $agents = [];
  
  public function __construct(
    ClientInterface $httpClient,
    ConfigFactoryInterface $configFactory
  ) {
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
  }
  
  public function getAgent(string $name): AgentInterface {
    if (!isset($this->agents[$name])) {
      throw new \InvalidArgumentException("Agent $name not found");
    }
    return $this->agents[$name];
  }
  
  public function executeTask(string $agent, array $task): array {
    $agent = $this->getAgent($agent);
    $task = new Task($task);
    $task->validate();
    
    try {
      $result = $agent->execute($task->toArray());
      $this->logTask($task, $result);
      return $result;
    } catch (\Exception $e) {
      $this->handleError($e, $task);
      throw $e;
    }
  }
  
  public function addAgent(AgentInterface $agent): void {
    $this->agents[$agent->getName()] = $agent;
  }
  
  private function logTask(Task $task, array $result): void {
    \Drupal::logger('bluefly')->info('Task executed: @task', [
      '@task' => json_encode([
        'id' => $task->id,
        'type' => $task->type,
        'result' => $result
      ])
    ]);
  }
}
```

2. Create an agent:

```php
// src/Plugin/Agent/ContentAgent.php
namespace Drupal\bluefly\Plugin\Agent;

use Drupal\bluefly\AgentInterface;
use Drupal\Core\Plugin\PluginBase;

/**
 * @Agent(
 *   id = "content",
 *   label = @Translation("Content Agent"),
 *   description = @Translation("Handles content-related tasks")
 * )
 */
class ContentAgent extends PluginBase implements AgentInterface {
  public function getName(): string {
    return 'content';
  }
  
  public function getCapabilities(): array {
    return [
      'generate' => [
        'description' => 'Generate content',
        'parameters' => [
          'type' => 'string',
          'prompt' => 'string'
        ]
      ],
      'summarize' => [
        'description' => 'Summarize content',
        'parameters' => [
          'content' => 'string',
          'length' => 'integer'
        ]
      ]
    ];
  }
  
  public function execute(array $input): array {
    $capability = $input['capability'] ?? 'generate';
    
    switch ($capability) {
      case 'generate':
        return $this->generateContent($input);
      case 'summarize':
        return $this->summarizeContent($input);
      default:
        throw new \InvalidArgumentException("Unknown capability: $capability");
    }
  }
  
  private function generateContent(array $input): array {
    // Implement content generation
    $prompt = $input['prompt'];
    $type = $input['type'];
    
    // Call LLM Platform API
    $response = $this->callApi('generate', [
      'prompt' => $prompt,
      'type' => $type
    ]);
    
    return [
      'success' => true,
      'content' => $response['content'],
      'metadata' => [
        'model' => $response['model'],
        'tokens' => $response['usage']
      ]
    ];
  }
}
```

### Controller Implementation

```php
// src/Controller/TaskController.php
namespace Drupal\bluefly\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\bluefly\Service\LLM PlatformServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

class TaskController extends ControllerBase {
  private LLM PlatformServiceInterface $blueflyService;
  
  public function __construct(LLM PlatformServiceInterface $blueflyService) {
    $this->blueflyService = $blueflyService;
  }
  
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('bluefly.service')
    );
  }
  
  public function execute(Request $request): JsonResponse {
    try {
      $content = json_decode($request->getContent(), true);
      
      if (!isset($content['agent'], $content['task'])) {
        throw new \InvalidArgumentException('Missing required parameters');
      }
      
      $result = $this->blueflyService->executeTask(
        $content['agent'],
        $content['task']
      );
      
      return new JsonResponse($result);
    } catch (\Exception $e) {
      return new JsonResponse([
        'error' => $e->getMessage()
      ], 400);
    }
  }
}
```

### Event Subscribers

```php
// src/EventSubscriber/ContentSubscriber.php
namespace Drupal\bluefly\EventSubscriber;

use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\bluefly\Service\LLM PlatformServiceInterface;

class ContentSubscriber implements EventSubscriberInterface {
  private LLM PlatformServiceInterface $blueflyService;
  
  public function __construct(LLM PlatformServiceInterface $blueflyService) {
    $this->blueflyService = $blueflyService;
  }
  
  public static function getSubscribedEvents(): array {
    return [
      'entity.insert' => 'onEntityInsert',
      'entity.update' => 'onEntityUpdate',
      'entity.delete' => 'onEntityDelete'
    ];
  }
  
  public function onEntityInsert(EntityInterface $entity): void {
    if ($entity->getEntityTypeId() === 'node') {
      $this->processContent($entity, 'insert');
    }
  }
  
  private function processContent(EntityInterface $entity, string $action): void {
    try {
      $this->blueflyService->executeTask('content', [
        'action' => $action,
        'entity' => [
          'type' => $entity->getEntityTypeId(),
          'id' => $entity->id(),
          'data' => $entity->toArray()
        ]
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('bluefly')->error('Error processing content: @error', [
        '@error' => $e->getMessage()
      ]);
    }
  }
}
```

## Testing

### Kernel Tests

```php
// tests/Kernel/LLM PlatformServiceTest.php
namespace Drupal\Tests\bluefly\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\bluefly\Service\LLM PlatformService;
use Drupal\bluefly\Plugin\Agent\ContentAgent;

class LLM PlatformServiceTest extends KernelTestBase {
  protected static $modules = ['bluefly', 'system', 'user'];
  
  private LLM PlatformService $service;
  
  protected function setUp(): void {
    parent::setUp();
    
    $this->service = $this->container->get('bluefly.service');
    $this->service->addAgent(new ContentAgent([], 'content', []));
  }
  
  public function testExecuteTask(): void {
    $result = $this->service->executeTask('content', [
      'capability' => 'generate',
      'type' => 'article',
      'prompt' => 'Test content'
    ]);
    
    $this->assertTrue($result['success']);
    $this->assertArrayHasKey('content', $result);
    $this->assertArrayHasKey('metadata', $result);
  }
}
```

### Functional Tests

```php
// tests/Functional/TaskControllerTest.php
namespace Drupal\Tests\bluefly\Functional;

use Drupal\Tests\BrowserTestBase;

class TaskControllerTest extends BrowserTestBase {
  protected static $modules = ['bluefly', 'rest', 'basic_auth'];
  
  protected function setUp(): void {
    parent::setUp();
    
    $this->drupalCreateUser(['execute bluefly tasks']);
  }
  
  public function testExecuteTask(): void {
    $this->drupalLogin($this->createUser(['execute bluefly tasks']));
    
    $response = $this->drupalPost('/api/bluefly/task/execute', 'application/json', [
      'agent' => 'content',
      'task' => [
        'capability' => 'generate',
        'type' => 'article',
        'prompt' => 'Test content'
      ]
    ]);
    
    $this->assertSession()->statusCodeEquals(200);
    $this->assertJson($response);
    
    $data = json_decode($response, true);
    $this->assertTrue($data['success']);
  }
}
```

## Best Practices

1. **Service Design**
   - Use dependency injection
   - Implement interfaces
   - Handle errors gracefully
   - Log important events
   - Cache when appropriate

2. **Agent Development**
   - Define clear capabilities
   - Validate inputs
   - Handle errors
   - Implement monitoring
   - Document behavior

3. **Security**
   - Validate permissions
   - Sanitize inputs
   - Use CSRF protection
   - Implement rate limiting
   - Log security events

4. **Performance**
   - Use caching
   - Optimize database queries
   - Implement batch processing
   - Monitor resource usage
   - Profile critical paths

## Related Documentation

- [Module Development](../development/modules)
- [API Integration](../development/api)
- [Testing Guide](../development/testing)
- [Security Guide](../operations/security)
